##
# .test.test_libregistry
##
import abc
from .. import libregistry
from .. import lib

def test_predefined_interface(test):
	R = libregistry.Registry()

	@R.interface('foo')
	class IFoo(metaclass=abc.ABCMeta):
		pass

	@R.implementation('foo')
	class MFoo(object):
		pass

	F=MFoo()
	test.fail_if_not_instance(F, MFoo)
	test.fail_if_not_instance(F, IFoo)
	test.fail_if_not_subclass(MFoo, IFoo)
	test.fail_if_not(R.isinstance(F, 'foo'))
	test.fail_if_not(R.isdefinition(IFoo, 'foo'))

	# check for mro walk
	class XFoo(MFoo):
		pass
	XF = XFoo()
	test.fail_if_not_instance(XF, MFoo) # sanity
	test.fail_if_not_instance(XF, IFoo)
	test.fail_if_not(R.isinstance(XF, 'foo'))

def test_no_interfaces(test):
	R = libregistry.Registry()

	# declare an implemenation without an interface
	@R.implementation('bar')
	class AFoo(object):
		pass

	# validate that it implements 'bar', wtf that is
	test.fail_if_not(R.isinstance(AFoo(), 'bar'))

	# Now, a subclass that doesn't specify an implementation
	class BFoo(AFoo):
		pass
	test.fail_if_not(R.isinstance(BFoo(), 'bar'))

	class CFoo(object):
		pass
	test.fail_if(R.isinstance(CFoo(), 'bar'))

def test_not_abcmeta(test):
	R = libregistry.Registry()
	def declare():
		@R.interface('foo')
		class Foo(object):
			pass
	# declaration of the interface attempts to register
	# Imp, but Foo is not an ABC.
	test.fail_if_not_raised(AttributeError, declare)

def test_analyze(test):
	R = libregistry.Registry()
	@R.interface('bar')
	class BInterface(metaclass = abc.ABCMeta):
		@abc.abstractproperty
		def bar(self) -> R.reference('foo'):
			pass

	@R.interface('foo')
	class FInterface(BInterface):
		@abc.abstractproperty
		def foo(self) -> R.reference('bar'):
			pass

	x = set(R.analyze(FInterface))
	test.fail_if_not_equal(x, set(('foo','bar')))

def test_mimic(test):
	R = libregistry.Registry()
	M = lib.Mimic('meh')
	test.fail_if_not(R.isinstance(M, 'meh'))

def test_query(test):
	R = libregistry.Registry()
	@R.interface('bar')
	class BInterface(metaclass = abc.ABCMeta):
		@abc.abstractproperty
		def bar(self) -> R.reference('foo'):
			pass

	@R.interface('foo')
	class FInterface(metaclass = abc.ABCMeta):
		@abc.abstractproperty
		def bar(self) -> R.reference('bar'):
			pass

	@R.implementation('bar')
	class Eek(object):
		@property
		@classmethod
		def foo(typ):
			return 'constant'

	@R.implementation('foo')
	class Fook(object):
		@property
		def bar(typ) -> R.reference('bar'):
			return Eek()

	@R.implementation('foo')
	class Failed(object):
		@property
		def bar(typ) -> R.reference('foo'):
			return Eek()

	r = R.query('foo', subqueries = dict(bar = 'bar'))
	test.fail_if_not_equal(list(r), [Fook])

if __name__ == '__main__':
	from dev import libtest; libtest.execmodule()
